@@ -94,6 +94,8 @@ const ModuleVersions LoaderVersionInfo = {
     ABI_FONT_VERSION
 };
 
+static int ModuleDuplicated[] = {};
+
 static void
 FreeStringList(char **paths)
 {
@@ -435,14 +437,11 @@ FindModule(const char *module, const char *dirname, const char **subdirlist,
 	   PatternPtr patterns)
 {
     char buf[PATH_MAX + 1];
-    char *dirpath = NULL;
     char *name = NULL;
-    int dirlen;
     const char **subdirs = NULL;
     const char **s;
 
-    dirpath = (char *)dirname;
-    if (strlen(dirpath) > PATH_MAX)
+    if (strlen(dirname) > PATH_MAX)
 	return NULL;
     
     subdirs = InitSubdirs(subdirlist);
@@ -450,17 +449,15 @@ FindModule(const char *module, const char *dirname, const char **subdirlist,
 	return NULL;
 
     for (s = subdirs; *s; s++) {
-	if ((dirlen = strlen(dirpath) + strlen(*s)) > PATH_MAX)
+	if ((strlen(dirname) + strlen(*s)) > PATH_MAX)
 	    continue;
-	strcpy(buf, dirpath);
+	strcpy(buf, dirname);
 	strcat(buf, *s);
         if ((name = FindModuleInSubdir(buf, module)))
             break;
     }
 
     FreeSubdirs(subdirs);
-    if (dirpath != dirname)
-	free(dirpath);
 
     return name;
 }
@@ -785,7 +782,6 @@ ModuleDescPtr
 DuplicateModule(ModuleDescPtr mod, ModuleDescPtr parent)
 {
     ModuleDescPtr ret;
-    int errmaj, errmin;
 
     if (!mod)
 	return NULL;
@@ -794,14 +790,11 @@ DuplicateModule(ModuleDescPtr mod, ModuleDescPtr parent)
     if (ret == NULL)
 	return NULL;
 
-    if (!(ret->handle = LoaderOpen(mod->path, &errmaj, &errmin))) {
-        free(ret);
-        return NULL;
-    }
+    ret->handle = mod->handle;
 
     ret->SetupProc = mod->SetupProc;
     ret->TearDownProc = mod->TearDownProc;
-    ret->TearDownData = NULL;
+    ret->TearDownData = ModuleDuplicated;
     ret->child = DuplicateModule(mod->child, ret);
     ret->sib = DuplicateModule(mod->sib, parent);
     ret->parent = parent;
@@ -918,7 +911,7 @@ doLoadModule(const char *module, const char *path, const char **subdirlist,
 	goto LoadModule_fail;
     }
     ret->handle = LoaderOpen(found, errmaj, errmin);
-    if (ret->handle < 0)
+    if (ret->handle == NULL)
 	goto LoadModule_fail;
     ret->path = strdup(found);
 
@@ -1072,11 +1065,16 @@ UnloadModuleOrDriver(ModuleDescPtr mod)
     if (mod == NULL || mod->name == NULL)
 	return;
 
-    xf86MsgVerb(X_INFO, 3, "UnloadModule: \"%s\"\n", mod->name);
+    if (mod->parent)
+	xf86MsgVerb(X_INFO, 3, "UnloadSubModule: \"%s\"\n", mod->name);
+    else
+	xf86MsgVerb(X_INFO, 3, "UnloadModule: \"%s\"\n", mod->name);
 
-    if ((mod->TearDownProc) && (mod->TearDownData))
-	mod->TearDownProc(mod->TearDownData);
-    LoaderUnload(mod->name, mod->handle);
+    if (mod->TearDownData != ModuleDuplicated) {
+	if ((mod->TearDownProc) && (mod->TearDownData))
+	    mod->TearDownProc(mod->TearDownData);
+	LoaderUnload(mod->name, mod->handle);
+    }
 
     if (mod->child)
 	UnloadModuleOrDriver(mod->child);
@@ -1092,23 +1090,11 @@ UnloadSubModule(pointer _mod)
 {
     ModuleDescPtr mod = (ModuleDescPtr)_mod;
 
-    if (mod == NULL || mod->name == NULL)
+    /* Some drivers are calling us on built-in submodules, ignore them */
+    if (mod == (ModuleDescPtr)1)
 	return;
-
-    xf86MsgVerb(X_INFO, 3, "UnloadSubModule: \"%s\"\n", mod->name);
-
-    if ((mod->TearDownProc) && (mod->TearDownData))
-	mod->TearDownProc(mod->TearDownData);
-    LoaderUnload(mod->name, mod->handle);
-
     RemoveChild(mod);
-
-    if (mod->child)
-	UnloadModuleOrDriver(mod->child);
-
-    free(mod->path);
-    free(mod->name);
-    free(mod);
+    UnloadModuleOrDriver(mod);
 }
 
 static void
@@ -1135,6 +1121,7 @@ RemoveChild(ModuleDescPtr child)
     }
     if (mdp == child)
 	prevsib->sib = child->sib;
+    child->sib = NULL;
     return;
 }
 
