@@ -113,8 +113,8 @@ xf86InitHardwareCursor(ScreenPtr pScreen, xf86CursorInfoPtr infoPtr)
 void
 xf86SetCursor(ScreenPtr pScreen, CursorPtr pCurs, int x, int y)
 {
-    xf86CursorScreenPtr ScreenPriv =
-	pScreen->devPrivates[xf86CursorScreenIndex].ptr;
+    xf86CursorScreenPtr ScreenPriv = (xf86CursorScreenPtr)dixLookupPrivate(
+	&pScreen->devPrivates, xf86CursorScreenKey);
     xf86CursorInfoPtr infoPtr = ScreenPriv->CursorInfoPtr;
     unsigned char *bits;
 
@@ -123,7 +123,7 @@ xf86SetCursor(ScreenPtr pScreen, CursorPtr pCurs, int x, int y)
 	return;
     }
 
-    bits = pCurs->devPriv[pScreen->myNum];
+    bits = dixLookupScreenPrivate(&pCurs->devPrivates, CursorScreenKey, pScreen);
 
     x -= infoPtr->pScrn->frameX0 + ScreenPriv->HotX;
     y -= infoPtr->pScrn->frameY0 + ScreenPriv->HotY;
@@ -133,7 +133,7 @@ xf86SetCursor(ScreenPtr pScreen, CursorPtr pCurs, int x, int y)
 #endif
     if (!bits) {
 	bits = (*infoPtr->RealizeCursor)(infoPtr, pCurs);
-	pCurs->devPriv[pScreen->myNum] = bits;
+	dixSetScreenPrivate(&pCurs->devPrivates, CursorScreenKey, pScreen, bits);
     }
 
     if (!(infoPtr->Flags & HARDWARE_CURSOR_UPDATE_UNHIDDEN))
@@ -157,8 +157,8 @@ xf86SetCursor(ScreenPtr pScreen, CursorPtr pCurs, int x, int y)
 void
 xf86SetTransparentCursor(ScreenPtr pScreen)
 {
-    xf86CursorScreenPtr ScreenPriv =
-	pScreen->devPrivates[xf86CursorScreenIndex].ptr;
+    xf86CursorScreenPtr ScreenPriv = (xf86CursorScreenPtr)dixLookupPrivate(
+	&pScreen->devPrivates, xf86CursorScreenKey);
     xf86CursorInfoPtr infoPtr = ScreenPriv->CursorInfoPtr;
 
     if (!ScreenPriv->transparentData)
@@ -178,8 +178,8 @@ xf86SetTransparentCursor(ScreenPtr pScreen)
 void
 xf86MoveCursor(ScreenPtr pScreen, int x, int y)
 {
-    xf86CursorScreenPtr ScreenPriv =
-	pScreen->devPrivates[xf86CursorScreenIndex].ptr;
+    xf86CursorScreenPtr ScreenPriv = (xf86CursorScreenPtr)dixLookupPrivate(
+	&pScreen->devPrivates, xf86CursorScreenKey);
     xf86CursorInfoPtr infoPtr = ScreenPriv->CursorInfoPtr;
 
     x -= infoPtr->pScrn->frameX0 + ScreenPriv->HotX;
@@ -191,8 +191,8 @@ xf86MoveCursor(ScreenPtr pScreen, int x, int y)
 void
 xf86RecolorCursor(ScreenPtr pScreen, CursorPtr pCurs, Bool displayed)
 {
-    xf86CursorScreenPtr ScreenPriv =
-	pScreen->devPrivates[xf86CursorScreenIndex].ptr;
+    xf86CursorScreenPtr ScreenPriv = (xf86CursorScreenPtr)dixLookupPrivate(
+	&pScreen->devPrivates, xf86CursorScreenKey);
     xf86CursorInfoPtr infoPtr = ScreenPriv->CursorInfoPtr;
 
 #ifdef ARGB_CURSOR
@@ -247,7 +247,7 @@ RealizeCursorInterleave0(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     int words = size / (CUR_BITMAP_SCANLINE_PAD / 4);
 
 
-    if (!(mem = xcalloc(1, size)))
+    if (!(mem = calloc(1, size)))
 	return NULL;
 
     if (pCurs == NullCursor) {
@@ -255,7 +255,7 @@ RealizeCursorInterleave0(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
 	    DstM = (SCANLINE*)mem;
 	    if (!(infoPtr->Flags & HARDWARE_CURSOR_SWAP_SOURCE_AND_MASK))
 		DstM += words;
-	    (void)memset(DstM, -1, words * sizeof(SCANLINE));
+	    memset(DstM, -1, words * sizeof(SCANLINE));
 	}
 	return mem;
     }
@@ -354,8 +354,8 @@ RealizeCursorInterleave1(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     if (!(mem2 = RealizeCursorInterleave0(infoPtr, pCurs)))
 	return NULL;
 
-    if (!(mem = xcalloc(1, size))) {
-	xfree(mem2);
+    if (!(mem = calloc(1, size))) {
+	free(mem2);
 	return NULL;
     }
 
@@ -379,7 +379,7 @@ RealizeCursorInterleave1(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     }
 
     /* Free the uninterleaved cursor */
-    xfree(mem2);
+    free(mem2);
 
     return mem;
 }
@@ -397,8 +397,8 @@ RealizeCursorInterleave8(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     if (!(mem2 = RealizeCursorInterleave0(infoPtr, pCurs)))
 	return NULL;
 
-    if (!(mem = xcalloc(1, size))) {
-	xfree(mem2);
+    if (!(mem = calloc(1, size))) {
+	free(mem2);
 	return NULL;
     }
 
@@ -414,7 +414,7 @@ RealizeCursorInterleave8(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     }
 
     /* Free the uninterleaved cursor */
-    xfree(mem2);
+    free(mem2);
 
     return mem;
 }
@@ -432,8 +432,8 @@ RealizeCursorInterleave16(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     if (!(mem2 = RealizeCursorInterleave0(infoPtr, pCurs)))
 	return NULL;
 
-    if (!(mem = xcalloc(1, size))) {
-	xfree(mem2);
+    if (!(mem = calloc(1, size))) {
+	free(mem2);
 	return NULL;
     }
 
@@ -449,7 +449,7 @@ RealizeCursorInterleave16(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     }
 
     /* Free the uninterleaved cursor */
-    xfree(mem2);
+    free(mem2);
 
     return mem;
 }
@@ -467,8 +467,8 @@ RealizeCursorInterleave32(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     if (!(mem2 = RealizeCursorInterleave0(infoPtr, pCurs)))
 	return NULL;
 
-    if (!(mem = xcalloc(1, size))) {
-	xfree(mem2);
+    if (!(mem = calloc(1, size))) {
+	free(mem2);
 	return NULL;
     }
 
@@ -484,7 +484,7 @@ RealizeCursorInterleave32(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     }
 
     /* Free the uninterleaved cursor */
-    xfree(mem2);
+    free(mem2);
 
     return mem;
 }
@@ -502,8 +502,8 @@ RealizeCursorInterleave64(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     if (!(mem2 = RealizeCursorInterleave0(infoPtr, pCurs)))
 	return NULL;
 
-    if (!(mem = xcalloc(1, size))) {
-	xfree(mem2);
+    if (!(mem = calloc(1, size))) {
+	free(mem2);
 	return NULL;
     }
 
@@ -521,7 +521,7 @@ RealizeCursorInterleave64(xf86CursorInfoPtr infoPtr, CursorPtr pCurs)
     }
 
     /* Free the uninterleaved cursor */
-    xfree(mem2);
+    free(mem2);
 
     return mem;
 }
