@@ -23,17 +23,20 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
+
+#define Cursor Mac_Cursor
+#define BOOL Mac_BOOL
 #include <OpenGL/OpenGL.h>
 #include <OpenGL/gl.h>
 #include <OpenGL/glu.h>
 #include <OpenGL/glext.h>
 #include <ApplicationServices/ApplicationServices.h>
+#undef Cursor
+#undef BOOL
 
 #include "capabilities.h"
 
-#define Cursor X_Cursor
 #include "os.h"
-#undef Cursor
 
 static void handleBufferModes(struct glCapabilitiesConfig *c, GLint bufferModes) {
     if(bufferModes & kCGLStereoscopicBit) {
@@ -507,7 +510,7 @@ bool getGlCapabilities(struct glCapabilities *cap) {
 
 	err = CGLQueryRendererInfo((GLuint)-1, &info, &numRenderers);
     if(err) {
-	    fprintf(stderr, "CGLQueryRendererInfo error: %s\n", CGLErrorString(err));
+	    ErrorF("CGLQueryRendererInfo error: %s\n", CGLErrorString(err));
         return err;
 	}
 
@@ -518,15 +521,14 @@ bool getGlCapabilities(struct glCapabilities *cap) {
 
 	    err = handleRendererDescriptions(info, r, &tmpconf);
 	    if(err) {
-            fprintf(stderr, "handleRendererDescriptions returned error: %s\n", CGLErrorString(err));
-            fprintf(stderr, "trying to continue...\n");
+            ErrorF("handleRendererDescriptions returned error: %s\n", CGLErrorString(err));
+            ErrorF("trying to continue...\n");
             continue;
 	    }
 
 	    conf = malloc(sizeof(*conf));
 	    if(NULL == conf) {
-            perror("malloc");
-            OsAbort();
+                FatalError("Unable to allocate memory for OpenGL capabilities\n");
 	    }
 
 	    /* Copy the struct. */
